## Title: gniputil.py
## Author: Scott O. Nelson (SirGnip)
## License: LGPL (c) 2007 Scott O. Nelson
## Description: General purpose utilities

import galcon
import bot
import random
rr = random.randrange
import gnipoptions
import gnipmath

########################################
######################################## Misc utils
########################################
def GetLeader(planets):
    ''' returns the non-neutral user who is the leader (in terms of total production)'''
    countDict = {}
    for p in planets.values():
        # skip neutral
        if p.user.n == 0:
            continue
        if countDict.has_key(p.user):
            countDict[p.user] += p.production
        else:
            countDict[p.user] = p.production
    maxUser, maxPlanets = None, 0
    for user in countDict:
        if countDict[user] > maxPlanets:
            maxUser, maxPlanets = user, countDict[user]
    return maxUser


def GetPlanetsInRadius(planets, point, radius):
    '''This is a simple contenience routine that returns all planets who have their
    centers within the given radius from the given point'''
    found = []
    for p in planets.values():
        dist = gnipmath.Distance(point, p.pos)
        if dist < radius:
            found.append(p)
    return found
    
def GeneratePlayers(level):
    # set up the player
    u = galcon.Player()
    u.name = 'Player'
    level.users.add(u)
    level.player = u

    # set up the bots
    for botIdx in range(gnipoptions.Map.NumberOfBots):
        u = bot.Bot(level)
        u.name = 'Bot%d' % (botIdx + 1)
        level.users.add(u)


def GenerateRandomMap(level):
    # a list of home planets
    homes = []

    # set up the player's home planet
    u = level.users[1]
    p = galcon.Planet()
    p.production = gnipoptions.Map.HomeProduction
    p.ships = gnipoptions.Map.HomeShips
    p.user = u
    level.planets.add(p)
    homes.append(p)

    # Add each bot's home planet
    firstBotIndex = 2 # skip past Neutral and Player in user array
    for botIndex in range(firstBotIndex, len(level.users)):
        u = level.users[botIndex] 
        p = galcon.Planet()
        p.production = gnipoptions.Map.HomeProduction
        p.ships = gnipoptions.Map.HomeShips
        p.user = u
        level.planets.add(p)
        homes.append(p)

    # add random neutral planets
    for n in xrange(0, gnipoptions.Map.NeutralPlanets):
        p = galcon.Planet()
        p.production = rr(gnipoptions.Map.NeutralProductionRange[0], gnipoptions.Map.NeutralProductionRange[1])
        p.ships = rr(gnipoptions.Map.NeutralShipRange[0], gnipoptions.Map.NeutralShipRange[1])
        p.user = level.users[0] # neutral
        level.planets.add(p)
        
    return(homes)        



########################################
######################################## Color utils
########################################

def ColorDarken(color, delta):
    '''Linearly darkens up each of the three color components by the given
    value.  Clamps values to be valid color values (0-255)'''
    return((gnipmath.ClampLo(color[0] - delta, 0), gnipmath.ClampLo(color[1] - delta, 0), gnipmath.ClampLo(color[2] - delta, 0)))

def ColorLighten(color, delta):
    '''Linearly lightens up each of the three color components by the given
    value.  Clamps values to be valid color values (0-255)'''
    return((gnipmath.ClampHi(color[0] + delta, 255), gnipmath.ClampHi(color[1] + delta, 255), gnipmath.ClampHi(color[2] + delta, 255)))

def ColorLerp(c1, c2, u):
	'''linearly interpolate between two colors'''
	r = gnipmath.Lerp(c1[0], c2[0], u)
	g = gnipmath.Lerp(c1[1], c2[1], u)
	b = gnipmath.Lerp(c1[2], c2[2], u)
	return (r, g, b)



########################################
######################################## Actor Utils
########################################
class Actor:
    '''This is a template for what Actor-like objects should look like.  I
    could derive all my actors from this class, but I'm just relying on
    duck-typing so far.  This class is here mostly as an example of what
    Actor-like objects should look like.'''
    def __init__(self):
        assert False, 'Can not instantiate an Actor directly.  It is "abstract" and must be subclassed.'
        
    def loop(self):
        pass
    
    def paint(self, screen):
        pass
    
    def canreap(self):
        '''Queries if this actor can be destroyed?'''
        return(False)

    def reap(self):
        '''Marks this actor so that it can be reaped the next time possible.
        
        This method is not necessary for most Actor-like objects.  But, it is a
        common addtion if an Actor's lifetime needs to be cut short from an outside
        client.'''
        pass

        
class ActorList(list, Actor):
    '''Design goal of ActorLists:  easy access to individual items with the auto Step/Draw/Reap
    feature thrown in.  It is intended that a game would have multiple cActorLists.'''
    def loop(self):
        '''NOTE: Just because this class removes an actor from its list does not mean that
        actor will immediately be destroyed.  There can be other objects in the game
        that are still holding references to it.  So, this is not a guarantee the actor
        will be destroyed, just that it won't get Step() and Draw() called for it.'''
        actorsToDelete = []
        for actor in self:
            actor.loop()
            if actor.canreap():
                actorsToDelete.append(actor)
        for actorToDel in actorsToDelete:
            self.remove(actorToDel)

    def paint(self, screen):
        for actor in self:
            actor.paint(screen)



########################################
######################################## Mods
########################################

class PlanetRevoltMod:
    '''To implement, simply create and store a reference to this object in a Level object
    and then call loop() on it.

    TODO: don't revolt a planet if it is that player's only planet (can happen at the start of the game if the interval is short and no one has captured another planet yet
    '''
    def __init__(self, level, revoltInterval):
        self.enabled = gnipoptions.Revolt.Enabled
        if not self.enabled:
            return
        self.timer = 0
        self.revoltInterval = galcon.FPS*revoltInterval
        self.level = level
        self.drawTimer = 0
        self.revoltPlanet = None

    def loop(self):
        if not self.enabled:
            return
        if self.timer > self.revoltInterval:
            self.timer = 0
            # determine who is leading
            leaderUser = GetLeader(self.level.planets)
            # pick one of the leader's planets
            leaderPlanets = [p for p in self.level.planets.values() if p.user == leaderUser]
            self.revoltPlanet = random.choice(leaderPlanets)   
            # get random owner it will revolt to
            newUser = None
            while not newUser:
                newUser = random.choice(self.level.users)
                # skip if neutral or leader
                if newUser.n in (0, leaderUser.n):
                    newUser = None
            # set new owner
            self.revoltPlanet.user = newUser
            self.drawTimer = 30
        else:
            self.timer += 1

    def paint(self, screen):
        ''' draw visual indicator around the planet that revolted'''
        if not self.enabled:
            return
        if self.drawTimer > 0:
            screen.circle((128, 128, 128), self.revoltPlanet.pos, (80-(self.drawTimer*3.0)), 6)
            screen.circle((0, 0, 128), self.revoltPlanet.pos, (50-(self.drawTimer*1.5)), 6)
            self.drawTimer -= 1


class StatsLogger:
    '''Write out a comma separated file (.csv) that contains a log of the
    player's planet count, production and total fleet size.  This data is written
    out in a .csv format so it can easily be pulled into Excel for easy
    charting and Pivot Tables/Chart creation.

Not really a mod, but more a utility.  Every time a game of "SirGip_Revolt" is played,
stats of the game are logged to a file after the game is complete.  This log
file is saved in the "data" folder.  You can specify the filename in gnipoptions.py.
This file is a .csv that logs, per player, the number of plants owned, total production,
and total number of ships.  These values are logged every second.  So, the main idea
was to write this out in a format that would be easy to manipulate in Excel.  Excel
can open .csv files directly.  So, if you know how to do charts in Excel (or better yet,
Pivot Tables and Pivot Charts), you'll be able to very quickly got yourself a very
nice graphical representation of the ebb and flow of a Galcon game, suitable for framing.'''
    def __init__(self, level):
        self.enabled = gnipoptions.Stats.Enabled
        if not self.enabled:
            return
        self.level = level
        self.statsText = "Elapsed,PlayerName,Planets,TotalProd,TotalShips\n"
        self.statsTimer = 0
        self.statsSeconds = 0
        
    def DoStats(self):
        playerDict = {}
        for u in self.level.users.values():
            planets = 0
            prod = 0
            ships = 0
            for p in self.level.planets.values():
                if p.user == u:
                    planets += 1
                    prod += p.production
                    ships += p.ships
            self.statsText += '%d,%s,%d,%d,%d\n' % (self.statsSeconds, u.name, planets, prod, ships)

    def loop(self):
        if not self.enabled:
            return
        if self.statsTimer == 0:
            self.DoStats()
        self.statsTimer += 1
        if self.statsTimer == int(galcon.FPS):
            self.statsTimer = 0
            self.statsSeconds += 1

    def WriteStats(self):
        if not self.enabled:
            return
        galcon.file_save(gnipoptions.Stats.Filename + '.csv', self.statsText)
