## Title: symlevel.py
## Author: ThirdParty
## License: LGPL (c) 2007 "ThirdParty"
## Description: Level generator with symmetry and homeworld options.

import galcon
import random
rr = random.randrange
import math

import bots

import mclassic
import dialog
import mapfile

class Level_(mclassic.Level):
	def __init__(self,game,opts,parent,data = None,rotate = False):
		self.game = game
		self.opts = opts
		self.parent = parent
		self.data = data
		self.rot = rotate
		
	def init(self):
		# human player
		if self.opts['Human']:
			u = galcon.Player()
			u.name = 'Human'
			self.users.add(u)
			self.player = u
		else:
			u = galcon.Player()
			u.name = None
			self.users.add(u)
			self.player = u
			self.options.hide_stats = 0

		# bots
		for b in bots.BOTS:
			if self.opts[b[0]]:
				u = b[1](self)
				u.name = b[0]
				self.users.add(u)
			else:
				u = galcon.Bot()
				u.name = None
				self.users.add(u)
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)
			if self.rot:
				self.rotate()
				self.data = mapfile.tostring(self)
			
		self.sync()

		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def reset(self):
		return Level(self.game,self.opts,self.parent,self.data)
		
	def resetandrotate(self):
		return Level(self.game,self.opts,self.parent,self.data,True)
	
	def generate(self):
		SW,SH = galcon.SW,galcon.SH

		# a list of active players
		realplayers = []
		for u in self.users.values():
			if u.name != None and not isinstance(u, galcon.Neutral): realplayers.append(u)

		if self.opts['symmetry']: # symmetric generator
			# get some basic gist of the screen sizing ...
			sz = 16
			top = galcon.BORDER+sz
			left = galcon.PADDING+sz
			
			# add random planets
			numsets = int( (self.opts['planets']+len(realplayers)/2.0)/len(realplayers) )
			for n in xrange(0,numsets):
				set = []
				prod = rr(0,100)
				ships = rr(0,50)
				for u in realplayers:
					p = galcon.Planet()
					p.production = prod
					p.ships = ships
					p.user = self.users[0]
					self.planets.add(p)
					set.append(p)
				a = rr(0,360)
				rad = 0
				while rad < 16 or rad > SH/2 - sz:
					rad = math.hypot(SH/2 - rr(top,galcon.SH-top), SH/2 - rr(top,galcon.SH-top))
				inc = 360/len(set)
				for p in set:
					x = SW/2+(rad)*math.cos(a*math.pi*2.0/360.0)
					y = SH/2+(rad)*math.sin(a*math.pi*2.0/360.0)
					p.pos = x,y
					a += inc

			# add home planets
			random.shuffle(realplayers)
			homes = []
			for u in realplayers:
				for i in xrange(0,self.opts['homes']):
					p = galcon.Planet()
					p.production = self.opts['production']
					p.ships = self.opts['ships']
					p.user = u
					self.planets.add(p)
					homes.append(p)
			a = rr(0,360)
			inc = 360/len(homes)
			for p in homes:
				x = SW/2+(SH/2-sz)*math.cos(a*math.pi*2.0/360.0)
				y = SH/2+(SH/2-sz)*math.sin(a*math.pi*2.0/360.0)
				p.pos = x,y
				a += inc

			# add the hill if applicable
			if self.opts['hill']:
				p = galcon.Planet()
				p.production = self.opts['hillprod']
				p.ships = self.opts['hillships']
				p.user = self.users[0]
				p.name = 'the hill'
				self.planets.add(p)
				p.pos = galcon.SW/2,galcon.SH/2
			
			# set the scale
			self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
			
			# running this function syncs the data here with galcon
			# and gets valuable stuff like planet radius figured out, etc
			self.sync()

		else: # non-symmetric generator
			# a list of home planets
			homes = []
			for i in xrange(0,self.opts['homes']):
				for u in realplayers:
					p = galcon.Planet()
					p.production = self.opts['production']
					p.ships = self.opts['ships']
					p.user = u
					self.planets.add(p)
					homes.append(p)

			# add the hill if applicable
			hill = None
			if self.opts['hill']:
				p = galcon.Planet()
				p.production = self.opts['hillprod']
				p.ships = self.opts['hillships']
				p.user = self.users[0]
				p.name = 'the hill'
				self.planets.add(p)
				hill = p
			
			# add random planets
			for n in xrange(0,self.opts['planets']):
				p = galcon.Planet()
				p.production = rr(0,100)
				p.ships = rr(0,50)
				p.user = self.users[0]
				self.planets.add(p)
			
			# set the scale - this formula works good
			self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
			
			# running this function syncs the data here with galcon
			# and gets valuable stuff like planet radius figured out, etc
			self.sync()
			
			# get some basic gist of the screen sizing ...
			sz = 16
			top = galcon.BORDER+sz
			left = galcon.PADDING+sz
	
			# randomize the position of the planets
			for p in self.planets.values():
				p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

			# position the hill if applicable:
			if self.opts['hill']:
				hill.pos = SW/2,SH/2

			# place the home planets in a circular formation
			random.shuffle(homes)
			a = rr(0,360)
			inc = 360/len(homes)
			for p in homes:
				x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
				y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
				p.pos = x,y
				a += inc

	def rotate(self):
		realplayers = []
		for u in self.users.values():
			if u.name != None and not isinstance(u, galcon.Neutral): realplayers.append(u)
		numplayers = len(realplayers)
		for i in xrange(0,numplayers):
			realplayers[i].nextplayer = realplayers[(i+1)%numplayers]

		for p in self.planets.values():
			if not isinstance(p.user, galcon.Neutral): p.user = p.user.nextplayer

def Level(game,parent,opts,data=None,rotate = False):
	l_ = Level_(game,parent,opts,data,rotate)
	l = galcon.Level(l_)
	l_.state = l
	return l
	