## Title: Bot Wars (w/ Tournament Mode) Level code
## Author: Phil Hassey / ThirdParty
## License: LGPL (c) 2007 "ThirdParty"

import galcon
import random
rr = random.randrange
import math

import classic
import dialog
import mapfile

import gui

class Level_(classic.Level):
	def __init__(self,game,opts,parent,order = None,data = None):
		self.game = game
		self.opts = opts
		self.parent = parent
		self.order = order
		self.data = data
		
	def init(self):
		self.font = galcon.Font(10)
		
		if self.order == None:
			bs = self.game.bots[:]
			random.shuffle(bs)
			self.bots = []
			for bot in bs[:self.opts['bots']]:
				u = bot.klass(self)
				u.name = bot.name
				u.bot = bot
				bot.user = u
				self.bots.append(u)
				self.users.add(u)
		else:
			self.bots = []
			for i in xrange(0,self.opts['bots']):
				bot = self.game.unsortedbots[self.order[i]]
				u = bot.klass(self)
				u.name = bot.name
				u.bot = bot
				bot.user = u
				self.bots.append(u)
				self.users.add(u)
		
		u = galcon.Player()
		u.name = 'player'
		self.player = u
		self.users.add(u)

		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
			self.parent.data = self.data
		else:
			mapfile.fromstring(self,self.data)

		self.options.hide_stats = 0
		self.options.percent = False # get rid of Ships % display
		self.sync()
			
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def generate(self):
		# a list of home planets
		homes = []

		a = self.opts['production']
		if self.opts['variation']:
			a += rr(-50,50)
			if a < 15: a = 15
			elif a > 200: a = 200
		b = self.opts['ships']
		if self.opts['variation']:
			b += rr(-50,50)
			if b < 15: a = 15
			elif b > 200: a = 200
		
		for i in xrange(0,self.opts['homes']):
			for u in self.bots:
				# set up the bots' home planets
				p = galcon.Planet()
				p.production = a
				p.ships = b
				p.user = u
				self.planets.add(p)
				homes.append(p)

		a = self.opts['planets']
		if self.opts['variation']:
			a += rr(-15,15)
			if a < 10: a = 10
			elif a > 65: a = 65
		
		# add the random planets
		for n in xrange(0,a):
			p = galcon.Planet()
			p.production = rr(0,100)
			p.ships = rr(0,50)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()
		
		# get some basic gist of the screen sizing ...
		sz = 16
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz

		# randomize the position of the planets
		for p in self.planets.values():
			p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

		# place the home planets in a circular formation
		SW,SH = galcon.SW,galcon.SH
		random.shuffle(homes)
		a = rr(0,360)
		inc = 360/len(homes)
		for p in homes:
			x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
			y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
			p.pos = x,y
			a += inc
			
	def loop(self):
		self.frame += 1
		
		for u in self.users.values():
			u.loop()
		
		for p in self.planets.values():
			if p.user.n == 0: continue
			p.ships += p.production / (galcon.FPS*50.0)
			
		if self.is_over():
			self.stop()
			return self.parent
		
	def is_over(self):
		# if we've timed out, it's all over
		if self.opts['timeout'] and self.frame > (self.opts['timeout']*FPS*60):
			return True
	
		# count up active items per user
		self.calc_stats()
		
		# if more than one bot has active items, we're still live
		live = 0
		for u in self.bots:
			if u.s_planets != 0 or u.s_fleets != 0: live += 1
		if live < 2:
			return True
		
	def calc_stats(self):
		for u in self.users.values():
			u.s_planets = 0
			u.s_production = 0
			u.s_fleets = 0
			u.s_ships = 0
		for p in self.planets.values():
			if isinstance(p.user,galcon.Neutral): continue
			p.user.s_planets += 1
			p.user.s_production += p.production
			p.user.s_ships += p.ships 
		for f in self.fleets.values():
			if isinstance(f.user,galcon.Neutral): continue
			f.user.s_fleets += 1
			f.user.s_ships += f.ships
		
	def stop(self):
		# everyone gets a round!
		self.game.rounds += 1
		for u in self.bots:
			bot = u.bot
			bot.rounds += 1
			
		# count up active items per user
		self.calc_stats()
		
		# figure out who the winner is !
		def by_winner(a,b):
			# basically, by production ..
			# but fall back to ships
			# though if someone has 1000000 ships that'll count a bit extra
			# not very likely with a 3 minute limit though :)
			p = (b.s_production - a.s_production)
			s = (b.s_ships - a.s_ships)
			if p * 1000000 + s > 0: return 1
			elif p * 1000000 + s < 0: return -1
			else: return 0
#			return int(round(p * 1000000 + s))
		self.bots.sort(by_winner)
		win = self.bots[0]
		
		# hand out the points ...
		for u in self.bots:
			bot = u.bot
			if u == win:
				bot.win += 1
				bot.points += (len(self.bots)-1)
				bot.total += (len(self.bots)-1)
			else:
				bot.lose += 1
				bot.total += 1
		
			
	def paint(self,screen):
		if self.first:
			self.first = False
			
		x,y = 0,0
		fnt = self.font
		t = gui.Table(style={'margin':8})
		
		t.tr()
		tm = self.frame/FPS
		if not self.opts['tourney']:
			t.td(gui.Label('Round %s - %d:%02d'%(self.game.rounds+1,tm/60,tm%60),style={'margin-bottom':8}),colspan=5)
		else:
			t.td(gui.Label('Round %s, Game %s - %d:%02d'%(self.game.sets+1,self.game.rounds+1,tm/60,tm%60),style={'margin-bottom':8}),colspan=5)
		
		bs = [u.bot for u in self.bots]
		for bot in self.game.bots:
			bot.percent = int(round(bot.points*100.0/max(1,bot.total)))
		def bypercent(a,b): return b.percent-a.percent
		self.game.bots.sort(bypercent)
		for bot in self.game.bots:
			c = (0xaa,0xaa,0xaa)
			if bot in bs: c = bot.user.color
			#img = fnt.render('%02d - %s - %d'%(round(bot.win*100.0/max(1,bot.total)),bot.name,bot.rounds),1,c)
			#screen.blit(img,(x,y));y += img.get_height()
			s = {'font':fnt,'color':c,'padding-left':2,'padding-right':2}
			t.tr()
			t.td(gui.Label('%02d'%bot.percent,style=s))
			t.td(gui.Label(bot.name,style=s,align=-1))
			t.td(gui.Label('%d'%bot.win,style=s))
			t.td(gui.Label('%d'%bot.lose,style=s))
			#t.td(gui.Label('%d'%bot.draw,style=s))
			t.td(gui.Label('%d'%bot.rounds,style=s))
		
		#gui.Box(t,align=-1,valign=1,width=SW,height=SH).paint(screen)
		t.paint(screen)
		
		fnt = gui.theme['label']['font']
		img = fnt.render('current speed: %d'%self.opts['speed'],1,(255,255,255))
		screen.blit(img,((SW-img.get_width())/2,SH-BORDER+(BORDER-img.get_height())/2))

	def reset(self):
		return Level(self.game,self.opts,self.parent,self.order,self.data)

	def event(self,e):
		if e.type is KEYDOWN and e.key == K_ESCAPE:
			return dialog.Choice(self.game,None,[
				('continue',K_c,self.state),
				('restart',K_r,self.reset),
				('next',K_n,self.parent),
				('quit',K_q,False),
				])
				
		m = {
			K_1:1,
			K_2:2,
			K_3:3,
			K_4:4,
			K_5:6,
			K_6:8,
			K_7:12,
			K_8:16,
			K_9:24,
			K_0:32,
			}
			
		if e.type is KEYDOWN and e.key in m:
			self.opts['speed'] = m[e.key]
		if e.type is KEYDOWN and e.key in (K_LEFT,):
			self.opts['speed'] = max(1,min(FPS,self.opts['speed']-1))
		if e.type is KEYDOWN and e.key in (K_RIGHT,):
			self.opts['speed'] = max(1,min(FPS,self.opts['speed']+1))

def Level(game,opts,parent,order = None,data = None):
	l_ = Level_(game,opts,parent,order,data)
	l = galcon.Level(l_)
	
	# crazy speed hack
	loop_ = l.loop
	def loop():
		for n in xrange(0,opts['speed']):
			r = loop_()
			if r != None:
				return r
		#print l_.frame
	l.loop = loop

	l_.state = l
	return l
	