## Title: Map Editor
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: An easy to use Galcon map editor.

import galcon

import dialog
import mapfile
import level

HELP = [
        'Map Editor - Help',
        '(c) 2007 Phil Hassey',
        '',
        'F1 - Help | F9 - Reset | ESC - Quit',
        'F5 - Save | F6 - Play | F7 - Load',
        '',
        'Right Click / Insert - Add a planet',
        'Right Click (on a planet) / Delete - Remove a planet',
        'Scroll Wheel / Arrows - Adjust ships',
        'Shift + (Scroll Wheel / Arrows) - Adjust Production',
        '',
        'Number Keys - Change owner',
        'Letters / Backspace - Modify planet name',
        '',
        'Page up / Page down - Adjust scale',
        ]

class Edit_(galcon.State):
    def __init__(self,game,fname=None):
        self.game = game
        self.fname = fname
        self.hover = None
        self.cur = None
        self.next = None
        
    def init(self):
        # set up the player
        u = galcon.Player()
        u.name = 'player'
        self.users.add(u)
        self.player = u

        for n in xrange(2,13):
            # set up the bot
            u = galcon.Bot()
            u.name = 'bot'
            self.users.add(u)
            
        self.font = galcon.Font(15)
        
        self.options.hide_stats = 0
        self.options.scale = 1.0
        self.scale = 100
        self.options.percent = False # get rid of Ships % display
        
        if self.fname != '':
            self.do_load()

    def event(self,e):
        #check hovering ..
        if e.type == galcon.MOUSEMOTION:
            self.hover = None
            for p in self.planets.values():
                if p.rect.collidepoint(e.pos):
                    self.hover = p
            # drag and drop
            if self.cur != None and e.buttons[0]:
                self.cur.pos = e.pos
        
        # select an item
        if e.type is galcon.MOUSEBUTTONDOWN and e.button == 1:
            self.cur = self.hover
        
        # create / delete an item
        if e.type is galcon.MOUSEBUTTONDOWN and e.button == 3:
            # create a planet
            if self.hover == None:
                p = self.new_planet(e.pos)
                self.hover = self.cur = p
            # destroy a planet
            else:
                self.planets.remove(self.hover)
                self.hover = self.cur = None
            self.sync()
            
        # delete an item
        if e.type is galcon.KEYDOWN and e.key in (galcon.K_DELETE,):
            if self.cur != None:
                self.planets.remove(self.cur)
                self.hover = self.cur = None
            self.sync()
            
        # create an item
        if e.type is galcon.KEYDOWN and e.key in (galcon.K_INSERT,):
            p = self.new_planet(galcon.mouse_get_pos())
            self.cur = p
            self.sync()
            
        # increase / decrease stuff ...
        if self.cur:
            mods = galcon.key_get_mods()
            p = self.cur
            inc = 5
            if e.type is galcon.MOUSEBUTTONDOWN and e.button == 4:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            if e.type is galcon.KEYDOWN and e.key == galcon.K_UP:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            inc = -5
            if e.type is galcon.MOUSEBUTTONDOWN and e.button == 5:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            if e.type is galcon.KEYDOWN and e.key == galcon.K_DOWN:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            inc = 1
            if e.type is galcon.KEYDOWN and e.key == galcon.K_RIGHT:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            inc = -1
            if e.type is galcon.KEYDOWN and e.key == galcon.K_LEFT:
                if (mods&galcon.KMOD_SHIFT) == 0: p.ships += inc
                if (mods&galcon.KMOD_SHIFT) != 0: p.production += inc
            p.ships = max(0,min(1000,p.ships))
            p.production = max(0,min(200,p.production))
            
        # change the owner
        if self.cur != None and e.type is galcon.KEYDOWN and e.key in xrange(galcon.K_0,galcon.K_9+1):
            n = e.key - galcon.K_0
            if n in self.users:
                self.cur.user = self.users[n]
        if self.cur != None and e.type is galcon.KEYDOWN and e.key in xrange(galcon.K_KP0,galcon.K_KP9+1):
            n = e.key - galcon.K_KP0
            if n in self.users:
                self.cur.user = self.users[n]
                
        # edit the planet name
        if self.cur != None and e.type is galcon.KEYDOWN and e.key == galcon.K_BACKSPACE:
            self.cur.name = self.cur.name[:-1]
        if self.cur != None and e.type is galcon.KEYDOWN and (e.key in xrange(galcon.K_a,galcon.K_z+1)):
            self.cur.name += str(e.unicode)
            
        # change the scale ...
        if e.type is galcon.KEYDOWN and e.key == galcon.K_PAGEUP:
            self.scale += 10
            self.scale = min(200,self.scale)
            self.options.scale = self.scale/100.0
        if e.type is galcon.KEYDOWN and e.key == galcon.K_PAGEDOWN:
            self.scale -= 10
            self.scale = max(50,self.scale)
            self.options.scale = self.scale/100.0
            
        # various functions ..
        if e.type is galcon.KEYDOWN and e.key == galcon.K_F1:
            return dialog.Help(self.game,HELP,self.state)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_F5:
            return dialog.FileDialog(self.game,'Save As:',self.fname,self.save,self.state)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_F6:
            data = mapfile.tostring(self)
            return level.Level(self.game,data,self.state)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_F7:
            return dialog.FileDialog(self.game,'Open:',self.fname,self.load,self.state)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_F9:
            return dialog.Prompt(self.game,'Reset?',self.reset,self.state)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_ESCAPE:
            return dialog.Prompt(self.game,'Quit?',False,self.state)
            
        # we don't want any normal "player" interaction :)
        return True
        
    def reset(self):
        return Edit(self.game)
    
    def save(self,fname):
        self.fname = fname
        r = self.do_save()
        if r != None: return r
        return self.state
    
    def load(self,fname):
        r = self.check_load(fname)
        if r != None: return r
        return Edit(self.game,fname)
        
    def check_load(self,fname):
        if not galcon.file_check_name(fname):
            return dialog.Pause(self.game,'invalid file name: %s'%fname,self.state)
        if not galcon.file_check_exists(fname):
            return dialog.Pause(self.game,'file does not exist: %s'%fname,self.state)

    def do_load(self):
        fname = self.fname
        data = galcon.file_load(fname)
        mapfile.fromstring(self,data)
        
    def do_save(self):
        fname = self.fname
        if not galcon.file_check_name(fname):
            return dialog.Pause(self.game,'invalid file name: %s'%fname,self.state)
        data = mapfile.tostring(self)
        galcon.file_save(fname,data)
            
    def new_planet(self,pos):
        p = galcon.Planet()
        p.pos = pos
        p.production = 50
        p.ships = 50
        p.user = self.users[0]
        self.planets.add(p)
        return p

    
    def paint(self,screen):
        c = (255,255,255)
        pad = 4
        screen.line(c,(pad,BORDER),(SW-pad,BORDER),2)
        screen.line(c,(pad,SH-BORDER),(SW-pad,SH-BORDER),2)
        
        if self.hover != None:
            p = self.hover
            screen.circle(c,p.pos,p.r+pad,2)
    
        if self.cur != None:
            p = self.cur
            screen.circle(c,p.pos,p.r+pad,3)
            
        p = self.cur
        if self.hover: p = self.hover
        if p != None:
            tab = '    '
            text = 'player: %s'%p.user.name
            text += tab + 'planet: %s_'%p.name
            text += tab + 'production: %d'%p.production
            text += tab + 'ships: %s'%p.ships
            img = self.font.render(text,1,c)
            screen.blit(img,((SW-img.get_width())/2,(BORDER-img.get_height())/2))
            
        text = 'Map Edit (c) 2007 Phil Hassey - F1 for help'
        img = self.font.render(text,1,c)
        screen.blit(img,((SW-img.get_width())/2,SH-BORDER+(BORDER-img.get_height())/2))
        
        text = 'Scale: %d%%'%int(self.options.scale*100)
        img = self.font.render(text,1,c)
        screen.blit(img,((SW-img.get_width())-pad,SH-BORDER+(BORDER-img.get_height())/2))
        
        text = '%s'%self.fname
        img = self.font.render(text,1,c)
        screen.blit(img,(pad,SH-BORDER+(BORDER-img.get_height())/2))
            
    def loop(self):
        if self.next:
            r = self.next
            self.next = None
            return r
        
def Edit(game,fname=''):
    l_ = Edit_(game,fname)
    l = galcon.Level(l_)
    l_.state = l
    return l
    
